; Sensor-Farbe
Const COL_SENSOR_R%	= 0
Const COL_SENSOR_G%	= 255
Const COL_SENSOR_B%	= 0

; Sandbox (in der eigentlich alles stattfindet)
Type TSandbox
	; Kartendaten
	Field map%[TILES_X * TILES_Y]
	; Kartenbild
	Field imgmap%
	; Kartentiles
	Field imgtiles%
	; Ziel
	Field goal.TGoal
	; Roboter
	Field robot.TRobot
	; Sensor-Daten
	Field dists#[SENSORS_VIS_CNT]
	; Startfeld
	Field startx#, starty#, startrot#
End Type
; Sandbox erstellen
Function CreateSandbox.TSandbox()
	Local sandbox.TSandbox = New TSandbox
	; Ziel und Roboter initialisieren
	sandbox\goal	= CreateGoal()
	sandbox\robot	= CreateRobot()
	Return sandbox
End Function
; Karte laden
Function LoadMap(pSandbox.TSandbox, pFile$)
	; Datei laden
	Local file% = ReadFile(pFile)
	Local x%, y%
	; ... und einlesen
	While Not Eof(file)
		Local txt$ = ReadLine(file)
		Local ln% = Len(txt) -1
		; Zeile einlesen
		For x = 0 To ln
			Local char% = Asc(Mid(txt,x+1,1))
			; "#" stellt eine Wand dar
			If char = Asc("#")
				pSandbox\map[TILES_X *y +x] = TILE_WALL
			EndIf
			; Die Zahlen 0 bis 3 stellen Ziel / Startposition dar
			Local c%
			For c = 0 To 3
				If char = (c +48) Then
					; Ziellinie setzen
					pSandbox\map[TILES_X *y +x]	= c +1
					pSandbox\goal\dir	= c
					pSandbox\goal\x		= x
					pSandbox\goal\y		= y
					; Roboter auf Start setzen
					pSandbox\robot\posx	= x +0.5
					pSandbox\robot\posy	= y +0.5
					pSandbox\robot\rot	= c *90
					; Startposition fr spteren gebrauch (Neustart) merken
					pSandbox\startx		= pSandbox\robot\posx
					pSandbox\starty		= pSandbox\robot\posy
					pSandbox\startrot	= pSandbox\robot\rot
				EndIf
			Next
		Next
		y = y +1
	Wend
	; Karte auf Bild zeichnen
	RenderMap(pSandbox)
End Function
; Karten-Tiles laden
Function LoadTiles(pSandbox.TSandbox, pFile$)
	pSandbox\imgtiles = LoadAnimImage(pFile$, TILE_SIZE, TILE_SIZE, 0, 6)
End Function
; Kartenbild erstellen
Function RenderMap(pSandbox.TSandbox)
	pSandbox\imgmap = CreateImage(TILES_X * TILE_SIZE, TILES_Y * TILE_SIZE)
	Local x%, y%
	For y = 0 To TILES_Y -1
		For x = 0 To TILES_X -1
			DrawImage(pSandbox\imgtiles, x * TILE_SIZE, y * TILE_SIZE, pSandbox\map[TILES_X *y +x])
		Next
	Next
	GrabImage(pSandbox\imgmap, 0, 0)
End Function

; Sandbox auf Startwerte zurcksetzen
Function ResetSandbox(pSandbox.TSandbox)
	pSandbox\robot\posx		= pSandbox\startx
	pSandbox\robot\posy		= pSandbox\starty
	pSandbox\robot\rot		= pSandbox\startrot
	pSandbox\robot\v1		= 0
	pSandbox\robot\v2		= 0
	pSandbox\robot\a1		= 0
	pSandbox\robot\a2		= 0
	ResetGoal(pSandbox\goal)
	UpdateSandbox(pSandbox)
End Function

; Sandbox aktualisieren
Function UpdateSandbox(pSandbox.TSandbox)
	; Roboter bewegen
	UpdateRobot(pSandbox\robot)
	UpdateSensors(pSandbox)
	; prfen, ob's irgendwo zur Kollision kommt
	Local robot.TRobot = pSandbox\robot
	Local cd#, cx#, cy#, cnx#, cny#, crot#
	Local collided%
	Local i%
	For i = 0 To SENSORS_CNT -1
		cd = robot\sensors[i]\dist - ROBOT_RADIUS
		; Kollision:
		If cd < 0 Then
			; Richtung der Kollision
			crot = robot\rot +i * SENSORS_DIVISION
			cnx = Cos(crot) * cd
			cny = Sin(crot) * cd
			; Abprallrichtung
			cx = cx + cnx
			cy = cy + cny
			collided = True
		EndIf
	Next
	; Kollision detektiert
	If collided Then
		; Roboter zurckstossen
		robot\posx = robot\posx + cx
		robot\posy = robot\posy + cy
		UpdateSensors(pSandbox)
	EndIf	
	CheckGoal(pSandbox\goal, pSandbox\robot)	
End Function

; Sensoren aktualisieren
Function UpdateSensors(pSandbox.TSandbox)
	Local robot.TRobot = pSandbox\robot
	Local i%, j%
	For i = 0 To SENSORS_CNT -1
		j = i + SENSORS_PER_SIDE
		If i > SENSORS_CNT / 2 Then
			j = j - SENSORS_CNT
		EndIf
		PositionSensor(robot\sensors[i], robot\posx, robot\posy, robot\rot +i * SENSORS_DIVISION)
		UpdateSensor(robot\sensors[i], pSandbox)
		If j >= 0 And j < SENSORS_VIS_CNT Then
			pSandbox\dists[j] = robot\sensors[i]\dist
		EndIf
	Next
End Function

; Sandbox zeichnen
Function DrawSandbox(pSandbox.TSandbox)
	; Karte
	DrawImage(pSandbox\imgmap, 0, 0)
	; Roboter
	DrawRobot(pSandbox\robot)
	; sensor-Schnittpunkte
	Color(COL_SENSOR_R, COL_SENSOR_G, COL_SENSOR_B)
	Local i%, j%, sensor.TSensor
	For j = -SENSORS_PER_SIDE To SENSORS_PER_SIDE
		i = j
		If i < 0 Then
			i = i + SENSORS_CNT
		EndIf
		sensor = pSandbox\robot\sensors[i]
		Rect(sensor\hitx * TILE_SIZE -1, sensor\hity * TILE_SIZE -1, 2,2)
	Next
End Function

;~IDEal Editor Parameters:
;~C#Blitz3D