' Roboter-Farbe
Const COL_ROBOT_R%	= 0
Const COL_ROBOT_G%	= 192
Const COL_ROBOT_B%	= 255
' "Augen"-Farbe
Const COL_EYE_R%	= 0
Const COL_EYE_G%	= 0
Const COL_EYE_B%	= 0

' Roboter
Type TRobot
	' Ort
	Field posx#, posy#, rot#
	' Beschleunigung der beiden Rder
	Field a1#, a2#
	' Geschwindigkeit der beiden Rder
	Field v1#, v2#
	' Sensoren
	Field sensors:TSensor[SENSORS_CNT]
End Type

' Roboter erstellen
Function CreateRobot:TRobot()
	Local robot:TRobot = New TRobot
	' Sensoren initialisieren
	Local i%
	For i = 0 To SENSORS_CNT -1
		robot.sensors[i] = New TSensor
	Next
	Return robot
End Function

' Beschleunigen
Function AccelerateRobot(pRobot:TRobot, pAcc1#, pAcc2#)
	pRobot.a1 = Min(Max(-MAX_ACC, pAcc1), MAX_ACC)
	pRobot.a2 = Min(Max(-MAX_ACC, pAcc2), MAX_ACC)
End Function
' Beschleunigen auf Ziel-Geschwindigkeit
Function AccelerateRobotTo(pRobot:TRobot, pVel1#, pVel2#)
	Local acc1# = pVel1 -pRobot.v1
	Local acc2# = pVel2 -pRobot.v2
	AccelerateRobot(pRobot, acc1, acc2)
End Function

' Geschwindigkeiten auslesen
Function GetV1#(pRobot:TRobot)
	Return pRobot.v1
End Function
Function GetV2#(pRobot:TRobot)
	Return pRobot.v2
End Function

' Roboterposition aktualisieren
Function UpdateRobot(pRobot:TRobot)
	Local v1#, v2#, rot#
	Local displacement#, dx#, dy#
	' Geschwindigkeit aktualisieren
	pRobot.v1 = Min(Max(-MAX_VEL, pRobot.v1 +pRobot.a1), MAX_VEL)
	pRobot.v2 = Min(Max(-MAX_VEL, pRobot.v2 +pRobot.a2), MAX_VEL)
	' Bewegung berechnen
	v1 = pRobot.v1
	v2 = pRobot.v2
	rot = pRobot.rot
	displacement = (v1 +v2) /2.0
	' Geradeaus fahren
	If v1 = v2 then
		dx = cos(rot) *displacement
		dy = sin(rot) *displacement
	' Kurve fahren
	Else
		Local vdiff# = v1 -v2
		Local alpha# = (vdiff *180.0) / (Pi * 2 * ROBOT_RADIUS)
		Local radius# = (displacement *180.0) / (Pi *alpha)
		Local du#, dv#
		du = sin(alpha) *radius
		dv = (1 -cos(alpha)) *radius
		dx = du *cos(rot) +dv *sin(rot)
		dy = du *sin(rot) -dv *cos(rot)
		pRobot.rot = pRobot.rot +alpha
	EndIf
	' Neue Position berechnen
	pRobot.posx = pRobot.posx +dx
	pRobot.posy = pRobot.posy +dy
End Function
' Roboter zeichnen
Function DrawRobot(pRobot:TRobot)
	Local x# = pRobot.posx * TILE_SIZE
	Local y# = pRobot.posy * TILE_SIZE
	Local r# = ROBOT_RADIUS * TILE_SIZE
	SetColor(COL_ROBOT_R, COL_ROBOT_G, COL_ROBOT_B)
	DrawOval(x-r, y-r, 2*r, 2*r)
	SetColor(COL_EYE_R, COL_EYE_G, COL_EYE_B)
	DrawOval(x +cos(pRobot.rot) *(r-4) -3, y +sin(pRobot.rot) *(r-4) -3, 5,5)
End Function

' Funktion frs manuelle Steuern des Roboters
Function MousePilotRobot(pRobot:TRobot)
	Local fw# = (-MouseY() + SCREEN_HEIGHT /2) /4000.0
	Local sw# = (MouseX() - SCREEN_WIDTH /2) /4000.0
	AccelerateRobotTo(pRobot, fw +sw, fw -sw)
End Function